﻿RectangleMorph subclass: #Breakout
	instanceVariableNames: 'scoreDisplayMorph lifeDisplayMorph gameFieldMorph levelCounter totalScore highscoreSaved'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!Breakout commentStamp: 'rkrk 2/1/2007 20:57' prior: 0!
This is a version of the famous game Breakout. We developed it as a project in software architecture class at the HPI in 2006/2007 (motto: GoodDesignNeedsTime ;-) ).

The game was first developed by Steve Wozniak and Nolan Bushnell in 1976 for ATARI,

The three background images which are used in the game are from hubblesite.org so they are credited to STScI and NASA. Thanks to everyone from these organizations for providing such beautiful images free in the internet!!

How to start:
choose "new morph..." -> "Breakout"
or "Breakout startNewGame"
or open the objects tool. You find Breakout in the games section.

How to play:
space - activate ball
left, right - move rackets horizontal
up, down - move rackets vertical
p - pause game

Developers:
- Robert Krahn
- Alexander Krasnogolowy
- Sebastian Wätzoldt

Have fun!!!


!Breakout methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:38'!
getButtonWithLabel: label selector: selector

	^ (IconicButton new
		target: self;
		actionSelector: selector;
		labelGraphic: (BreakoutResources getLabel: label);
		color: Color transparent;
		borderWidth: 0;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		yourself)! !

!Breakout methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:38'!
getRectangleForControls

	^ RectangleMorph new
		 layoutPolicy: TableLayout new;
		 listDirection: #leftToRight;
		 listCentering: #center;
		 hResizing: #spaceFill;
		 borderWidth: 0;
		 cellInset: 20;
		 color: Color transparent;
		 yourself
	! !

!Breakout methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:38'!
getTitle

	^ BreakoutResources getTitle! !

!Breakout methodsFor: 'initialization' stamp: 'rkrk 4/17/2007 02:39'!
initialize

	super initialize.
	totalScore := 0.
	highscoreSaved := false.	
	self 
		makeGameField;
		updateLifeDisplay;
		updateScoreDisplay;
		makeControlField;
		makeOwnLayout;
		makeBackground;
		startFromBeginning! !

!Breakout methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:38'!
makeBackground

	self fillStyle: (self class defaultFillStyleFor: self)
! !

!Breakout methodsFor: 'initialization' stamp: 'rkrk 4/17/2007 02:44'!
makeControlField

	self addMorph: 
		(self getRectangleForControls		
			addMorphFront: scoreDisplayMorph;
			addMorphFront: lifeDisplayMorph;
			addMorphBack: ((SketchMorph withForm: self getTitle)
								hResizing: #spaceFill;
								vResizing: #spaceFill;
								yourself);
			addMorphBack: (self getButtonWithLabel: 'retry' selector: #startFromBeginning"#nextLevel");
			addMorphBack: (self getButtonWithLabel: 'info' selector: #inform);
			addMorphBack: (self getButtonWithLabel: 'exit' selector: #end);
			yourself).! !

!Breakout methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:38'!
makeGameField	

	gameFieldMorph := BreakoutGameField new.
	self addMorphBack: gameFieldMorph.
	gameFieldMorph stop.
	ActiveHand newKeyboardFocus: gameFieldMorph.! !

!Breakout methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:38'!
makeOwnLayout

	self 
		useRoundedCorners;
		extent: Breakout defaultExtent;
		layoutPolicy: TableLayout new;
		layoutInset: 5;
		cellInset: 5;
		fullBounds! !


!Breakout methodsFor: 'user interaction' stamp: 'rkrk 4/17/2007 02:33'!
askUser: aQuestion

	|bq|
	bq := BreakoutQuestioner new question: aQuestion.
	gameFieldMorph addMorphCentered: bq.
	^ bq ask
! !


!Breakout methodsFor: 'gameFieldCollaboration' stamp: 'rkrk 4/14/2007 01:38'!
levelAccomplished

	totalScore := totalScore + gameFieldMorph score.
	self nextLevel.! !

!Breakout methodsFor: 'gameFieldCollaboration' stamp: 'rkrk 4/17/2007 02:26'!
levelFailed

	(self askUser: 'Play level again?')
		ifTrue:	[self loadLevel]
		ifFalse: [self addGameToHighscore].
! !

!Breakout methodsFor: 'gameFieldCollaboration' stamp: 'rkrk 4/14/2007 01:38'!
loadLevel

	gameFieldMorph	loadLevel: (BreakoutResources getLevel: levelCounter).
	! !

!Breakout methodsFor: 'gameFieldCollaboration' stamp: 'rkrk 4/14/2007 01:38'!
nextLevel

	levelCounter := levelCounter + 1.
	self loadLevel
	! !

!Breakout methodsFor: 'gameFieldCollaboration' stamp: 'rkrk 4/14/2007 01:38'!
startFromBeginning

	levelCounter := 1.
	self loadLevel
	! !

!Breakout methodsFor: 'gameFieldCollaboration' stamp: 'rkrk 4/17/2007 02:38'!
update

	highscoreSaved := false.
	
	self 
		updateLifeDisplay;
		updateScoreDisplay! !

!Breakout methodsFor: 'gameFieldCollaboration' stamp: 'rkrk 4/14/2007 01:38'!
updateLifeDisplay

	| form |
	form := (BreakoutResources getLifeDisplayLifeCount: gameFieldMorph life) form.
	lifeDisplayMorph 
		ifNil: [lifeDisplayMorph := SketchMorph withForm: form]
		ifNotNil: [lifeDisplayMorph form: form]! !

!Breakout methodsFor: 'gameFieldCollaboration' stamp: 'rkrk 4/17/2007 02:38'!
updateScoreDisplay	
	
	scoreDisplayMorph 
		ifNil: [scoreDisplayMorph := LedMorph new 
			string: '000000';
			color: (Color orange);
			hResizing: #shrinkWrap;
			vResizing: #shrinkWrap;
			yourself]
		ifNotNil: [| string |
			string := gameFieldMorph score asString.
			6 - string size timesRepeat: [string := '0' , string].
			scoreDisplayMorph string: string]! !


!Breakout methodsFor: 'highscore' stamp: 'rkrk 4/17/2007 02:41'!
addGameToHighscore

	| highscore |
	highscore := BreakoutResources getHighscore.
	highscore addHighScoreWithQuery: totalScore + gameFieldMorph score.
	gameFieldMorph addMorphCentered: highscore.
	BreakoutResources saveHighscore: highscore.
	highscoreSaved := true.! !


!Breakout methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:38'!
handlesMouseOver: evt

	^true! !

!Breakout methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:38'!
mouseEnter: evt

	evt hand newKeyboardFocus: gameFieldMorph! !


!Breakout methodsFor: 'closing' stamp: 'rkrk 4/17/2007 02:54'!
end

	gameFieldMorph stop.
	(highscoreSaved not and: [self askUser: 'Save highscore?'])
		ifTrue: [self addGameToHighscore. ^ self].
	self delete.
! !


!Breakout methodsFor: 'help' stamp: 'rkrk 4/14/2007 01:38'!
inform

	self inform: 'Controls:' , Character cr asString,
		'- Use the <LEFT>, <RIGHT>, <TOP> and <BOTTOM> arrows to control the rackets', 
		Character cr asString,
		'- Press <SPACE> to start the game and <P> to toggle pause' ,
		Character cr asString,Character cr asString! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Breakout class
	instanceVariableNames: ''!

!Breakout class methodsFor: 'default' stamp: 'rkrk 4/15/2007 23:13'!
defaultExtent
	^ 800 @ 600
	"^ 1200 @ 900"
	"^ World extent"! !

!Breakout class methodsFor: 'default' stamp: 'rkrk 1/31/2007 17:06'!
defaultFillStyleFor: aMorph
	^ ((GradientFillStyle ramp: {0.0 -> Color brown. 1.0 -> Color orange})
		origin: aMorph bounds origin;
		direction: 0 @ aMorph bounds height;
		yourself).
! !

!Breakout class methodsFor: 'default' stamp: 'rkrk 1/24/2007 17:06'!
defaultFont
	^ StrikeFont familyName: 'BitstreamVeraSans' size: 24! !

!Breakout class methodsFor: 'default' stamp: 'rkrk 1/16/2007 18:25'!
defaultFontBig
	^ StrikeFont familyName: 'BitstreamVeraSans' size: 80! !


!Breakout class methodsFor: 'parts bin' stamp: 'rkrk 1/17/2007 01:30'!
descriptionForPartsBin

	^ self
		partName: 'Breakout'
		categories: #('Games')
		documentation: 'The Game was created by Steve ''Woz'' Wozniak in 1976.'! !


Morph subclass: #BreakoutGameElement
	instanceVariableNames: 'speed direction moveProgress'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutGameElement commentStamp: 'rkrk 1/31/2007 01:02' prior: 0!
Abstract base class for all game elements. Common behavior like texturing or moving are defined here.!


!BreakoutGameElement methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
extent: newExtent 

	| oldDirection oldNormal newDirection newNormal |
	super extent: newExtent.
	
	"I changes the size of my fillstyle bitmap to fit the new extents of me."
	
	self fillStyle isBitmapFill ifFalse: [^ self].
	
	oldDirection := self fillStyle direction.
	oldNormal := self fillStyle normal.
	newDirection := (oldDirection * (newExtent / oldDirection r )) rounded.
	newNormal := (oldNormal * (newExtent / oldNormal r)) rounded.
	self fillStyle: (self fillStyle direction: newDirection;
		normal: newNormal;
		yourself).! !

!BreakoutGameElement methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
makeNoise! !


!BreakoutGameElement methodsFor: 'geometry' stamp: 'rkrk 4/14/2007 01:43'!
positionCentered: pos

	self position: pos - (self extent / 2)! !


!BreakoutGameElement methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:43'!
keyPressedCharacter: char! !

!BreakoutGameElement methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:43'!
keyReleased! !


!BreakoutGameElement methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
hit: aGameElement! !


!BreakoutGameElement methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
initialize

	super initialize.
	self 
		speed: 0;
		degree: 0;
		resetMoveProgress! !


!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/15/2007 23:25'!
calculateNewPosition

	self position: (self position translateBy: self samplingRate * self direction)! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
degree

	^ self direction degrees! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
degree: deg

	"My direction is a unit vector who has the given degree"
	self direction: (Point r: 1 degrees: deg)! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
direction

	^ direction! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
direction: newDir

	direction := newDir! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
getCollisions

	^ owner detectCollisions: self! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/15/2007 23:24'!
increaseMoveProgress

	"I count the move steps which are done in the #move: method for testing if I have finished my move"
	moveProgress := moveProgress + self samplingRate! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
move

	[self moveNotFinished]
		whileTrue: [self	move: self getCollisions].
	self	resetMoveProgress
		! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
move: aCollisionSet 

	"I send every gameElement which I hit a message. Then I change my position one step"
	aCollisionSet	do: [:element | element hit: self].
	self 
		calculateNewPosition;
		increaseMoveProgress! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/15/2007 23:49'!
moveNotFinished

	^ moveProgress < speed and: [owner notNil]! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
resetMoveProgress

	moveProgress := 0! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/16/2007 01:45'!
samplingRate

	^ 4! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/15/2007 23:26'!
speed

	^ speed! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/15/2007 23:28'!
speed: anInt 

	^ speed := anInt! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
speedToDefault

	self subclassResponsibility! !

!BreakoutGameElement methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
speedToZero

	self speed: 0! !


!BreakoutGameElement methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
intersects: aGameElement

	self == aGameElement ifTrue:[^ false].
	aGameElement isBreakoutGameField ifTrue: [^ aGameElement intersects: self].

	^ self bounds intersects: aGameElement bounds

	! !

!BreakoutGameElement methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
isBreakoutBall

	^ false! !

!BreakoutGameElement methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
isBreakoutBlock

	^ false! !

!BreakoutGameElement methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
isBreakoutGameField

	^ false! !

!BreakoutGameElement methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
isBreakoutRacket

	^ false! !


!BreakoutGameElement methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 01:43'!
setFillStyleFromForm: aForm

	self fillStyle: ((BitmapFillStyle form: aForm)
		origin: self topLeft;
		direction: self width @ 0;
		normal: 0 @ self height;
		yourself)! !

!BreakoutGameElement methodsFor: 'texturizing' stamp: 'rkrk 4/17/2007 02:12'!
texture: texturePath

	| aColorForm |
	aColorForm := BreakoutResources getTextureNamed: texturePath , 
													  FileDirectory slash , 
													  self textureName.
	aColorForm ifNotNil: [self setFillStyleFromForm: aColorForm]! !

!BreakoutGameElement methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 01:43'!
textureName

	self subclassResponsibility! !


!BreakoutGameElement methodsFor: 'printing' stamp: 'rkrk 4/14/2007 01:43'!
storeOn: aStream

	"remove the own texture to decrease the space wich is needed to store the object as a string"
	self useDefaultFill.
	super storeOn: aStream! !


BreakoutGameElement subclass: #BreakoutBall
	instanceVariableNames: 'isCaught'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutBall commentStamp: 'rkrk 1/31/2007 01:00' prior: 0!
The BreakoutBall is a Morph which has a specified radius. The direction of this ball is a normalized vector (length of vector  x + y is one). The ball bounces of from other gameElements.!


!BreakoutBall methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
radius

	^ self extent x / 2! !


!BreakoutBall methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
affectDirectionFrom: aGameElement	

	| deltaAngle |
	deltaAngle := (aGameElement degree = 0 or: [aGameElement degree = 270])
		ifTrue: [aGameElement speed]
		ifFalse: [aGameElement speed negated].
	self degree: self degree + deltaAngle! !

!BreakoutBall methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
bounceFromCorner: corner 

	"A tangent is applied to the corner and an angle of reflection is computed"
	| r |
	r := (corner - self center) degrees.
	(self degree - r) abs < 90
		ifTrue: [self degree: 2 * r - self degree - 180].! !

!BreakoutBall methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
bounceFromSide: aSide Point: aPoint

	| dir |
	dir := self direction.
	(#(#left #right) includes: aSide)
		ifTrue: [self center x < aPoint x
				ifTrue: [self direction: dir x abs negated @ dir y]
				ifFalse: [self direction: dir x abs @ dir y]].
	(#(#bottom #top) includes: aSide)
		ifTrue: [self center y < aPoint y
				ifTrue: [self direction: dir x @ dir y abs negated]
				ifFalse: [self direction: dir x @ dir y abs]].! !

!BreakoutBall methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
collisionWithGameElement: aGameElement

	"calculates the nearest point from the ball to the border of the rectangle
	and bounce from a side of the rectangle or from a corner"
	| distance boundingBox pointOnBoundingBox |
	boundingBox := aGameElement bounds.
	pointOnBoundingBox := boundingBox pointNearestTo: self center.
	distance := self center dist: pointOnBoundingBox.
	
	distance < self radius ifFalse: [^ self].
	
	(boundingBox corners includes: pointOnBoundingBox)
		ifTrue: [self bounceFromCorner: pointOnBoundingBox]
		ifFalse: [self bounceFromSide: (boundingBox sideNearestTo: pointOnBoundingBox) 
					 Point: pointOnBoundingBox.].
				
	isCaught ifFalse: [self affectDirectionFrom: aGameElement].
	aGameElement hit: self! !

!BreakoutBall methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
hit: aGameElement

	isCaught := true.
	aGameElement isBreakoutBall ifFalse: [
		[(owner isNil or: [self getCollisions isEmpty]) not]
			whileTrue: [self move: self getCollisions]].
	isCaught := false
		! !


!BreakoutBall methodsFor: 'initialization' stamp: 'rkrk 4/15/2007 23:35'!
initialize

	super initialize.
	isCaught := false.
	self 
		color: Color green;
		speedToDefault;
		degree: 360 atRandom;
		extent: 30 asPoint.! !


!BreakoutBall methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
degree: deg 

	"Sets a new diretion for the ball. If the new
	direction is 90, 180, 270... (+/- 10) the direction
	is corrected to the nearest legal degree. For
	example: degree: 87 will be corrected to 80,
	degree 183 will be corrected to 190"
	| delta normalized |
	delta := 0.
	normalized := deg \\ 90.
	normalized < 10
		ifTrue: [delta := 10 - normalized].
	normalized > 80
		ifTrue: [delta := 80 - normalized].
	super degree: deg + delta! !

!BreakoutBall methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
move: aCollisionSet

	aCollisionSet do: [:gameelement | (self collisionWithGameElement: gameelement)].
	self
		calculateNewPosition;
		increaseMoveProgress! !

!BreakoutBall methodsFor: 'moving' stamp: 'rkrk 4/15/2007 23:30'!
speedToDefault

	self speed: 4.! !


!BreakoutBall methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
isBreakoutBall

	^ true! !


!BreakoutBall methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 01:43'!
textureName

	^ 'ball.gif'! !


!BreakoutBall methodsFor: 'event handling' stamp: 'rkrk 4/4/2007 01:00'!
keyPressedCharacter: char

	(char = Character space or: [char = $1])
		ifTrue: [self speedToDefault]! !


BreakoutGameElement subclass: #BreakoutBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutBlock commentStamp: 'rkrk 1/31/2007 00:59' prior: 0!
Blocks react when they are hit with a Ball!
]style[(42)f1!


!BreakoutBlock methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 00:00'!
score

	^ 10! !


!BreakoutBlock methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 00:00'!
hit: aGameElement

	owner blockDestroyed: self.! !


!BreakoutBlock methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 00:00'!
initialize

	super initialize.
	self color: Color blue! !


!BreakoutBlock methodsFor: 'testing' stamp: 'rkrk 4/14/2007 00:00'!
isBreakoutBlock

	^ true! !

!BreakoutBlock methodsFor: 'testing' stamp: 'rkrk 4/14/2007 00:00'!
isDestroyable

	^ true! !


!BreakoutBlock methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 00:01'!
textureName

	^ 'normalBlock.gif'! !


!BreakoutBlock methodsFor: 'copying' stamp: 'rkrk 4/14/2007 00:00'!
copy

	^ self class new 
		fillStyle: self fillStyle;
		yourself! !


BreakoutBlock subclass: #BreakoutBallBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutBallBlock commentStamp: 'rkrk 1/31/2007 01:12' prior: 0!
Instances generate a ball.!
]style[(26)f1!


!BreakoutBallBlock methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 00:00'!
score

	^ 50! !


!BreakoutBallBlock methodsFor: 'collision handling' stamp: 'rkrk 4/13/2007 23:59'!
hit: aGameElement

	owner bonusBallAtPosition: self center.
	super hit: aGameElement! !


!BreakoutBallBlock methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 00:00'!
textureName

	^ 'ballBlock.gif'! !


BreakoutGameElement subclass: #BreakoutGameField
	instanceVariableNames: 'score criticalSides life textMorph level gameElements'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutGameField commentStamp: 'rkrk 1/31/2007 00:58' prior: 0!
Container class which is responsible for things like sending step messages, detectCollisions, adding/removing morphs!
]style[(116)f1!


!BreakoutGameField methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
gameElements

	^ gameElements! !


!BreakoutGameField methodsFor: 'bonus' stamp: 'rkrk 4/15/2007 23:52'!
bonusBallAtPosition: pos

	self addMorphBack: (level ball
		center: pos;
		degree: 360 atRandom;
		speedToDefault;
		yourself)! !


!BreakoutGameField methodsFor: 'collision handling' stamp: 'rkrk 4/4/2007 00:48'!
detectCollisions: aGameElement 	

	^ (self gameElements select: [:element | aGameElement intersects: element]) asSet.! !

!BreakoutGameField methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
hit: aGameElement 

	" Check for all sides if aGameElement has a distance equal or less than his own extent to this side. f this is true then remove this gameElement"
	self bounds forPoint: aGameElement center 
		closestSideDistLen: [:side :dist :sideLength | 
			((criticalSides includes: side) and: [dist <= aGameElement radius]) 
				ifTrue: [self gameElementHitsCriticalSide: aGameElement]]! !

!BreakoutGameField methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
intersects: aGameElement

	^(self bounds containsRect: aGameElement bounds) not! !


!BreakoutGameField methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:43'!
handlesKeyboard: evt

	^true! !

!BreakoutGameField methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:43'!
handlesMouseOver: evt

	^true! !

!BreakoutGameField methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:43'!
keyDown: evt 

	
	| char |
	char := evt keyCharacter.
	(#($P $p $#) includes: char) ifTrue: [self togglePause].
	self gameElements do: [:gameElement | gameElement keyPressedCharacter: char]! !

!BreakoutGameField methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:43'!
keyUp: evt 

	
	self gameElements do: [:gameElement | gameElement keyReleased]
	! !

!BreakoutGameField methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:43'!
mouseEnter: evt

	evt hand newKeyboardFocus: self! !


!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/3/2007 23:43'!
allBlocksDestroyed

	^(self gameElements noneSatisfy: [:gameElement | 
		gameElement isBreakoutBlock and: [gameElement isDestroyable]])
		
	! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
blockDestroyed: aBlock

	score _ score + aBlock score.
	owner update.
	self removeMorph: aBlock.
	self	allBlocksDestroyed ifTrue: [self levelFinishedSuccessfully: true].
	! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
containsMoreBalls

	^ self gameElements anySatisfy: [:gameElement | gameElement isBreakoutBall]! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
decreaseLife

	life := life - 1.
	owner update.
	life isZero
		ifTrue: [self gameOver]
		ifFalse: [self newBall]! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
gameElementHitsCriticalSide: aGameElement

	self removeMorph: aGameElement.
	self	containsMoreBalls ifFalse: [self decreaseLife]! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
gameOver

	self 
		stop;
		makeNoise;
		asGrayScale;
		removeAllMorphs;
		levelFinishedSuccessfully: false.! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
levelFinishedSuccessfully: aBool

	aBool 
		ifTrue:[owner levelAccomplished]
		ifFalse:[owner levelFailed]
	! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
life

	^ life! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
makeNoise

	(SampledSound soundNamed: 'laugh') play! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
newBall

	self	addMorphBack: (level ball 
		positionCentered: (self center);
		speedToZero;
		yourself)! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
score

	^ score! !

!BreakoutGameField methodsFor: 'gameContent' stamp: 'rkrk 4/14/2007 01:43'!
togglePause

	self isStepping
		ifTrue: [self stop; showText: 'PAUSE']
		ifFalse: [self showNothing; start]! !


!BreakoutGameField methodsFor: 'geometry' stamp: 'rkrk 4/14/2007 01:43'!
centerOfSide: side 

	^ self perform: (side asString , 'Center') asSymbol! !


!BreakoutGameField methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
initialize

	super initialize.
	self		
		color: Color black;
		reset;
		makeOwnLayout;
		makeTextMorph
	! !

!BreakoutGameField methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
makeOwnLayout.

	self 
		hResizing: #spaceFill;
		vResizing: #spaceFill.! !

!BreakoutGameField methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
makeTextMorph

	textMorph := StringMorph new
		contents: 'gameFieldInformation';
		font: Breakout defaultFontBig;
		color: Color red;
		yourself! !

!BreakoutGameField methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
reset

	self 
		stop;		
		removeAllMorphs;
		start.
	life := 5.
	score := 0.	! !


!BreakoutGameField methodsFor: 'levelLoading' stamp: 'rkrk 4/14/2007 01:43'!
addBlocks: newBlocks inColumns: columns andRows: rows 

	| raster |
	raster := Rasterizer
		rasterize: self bounds
		withColumns: columns
		andRows: rows.
	newBlocks with: (raster first: newBlocks size) do: [:block :position | 
		block ifNotNil:
			[self addMorphBack: (block 
				position: position;
				extent: raster cellWidth @ raster cellHeight;
				yourself)]]! !

!BreakoutGameField methodsFor: 'levelLoading' stamp: 'rkrk 4/14/2007 01:43'!
addRacketsAtCriticalSides: newCriticalSides 

	criticalSides := newCriticalSides.
	newCriticalSides do: 
			[:side | 
			| racket sideCenter |
			racket := level racketForSide: side.
			sideCenter := self centerOfSide: side.
			racket positionCentered: sideCenter + ((self center - sideCenter) * 0.15).
			self addMorphBack: racket]! !

!BreakoutGameField methodsFor: 'levelLoading' stamp: 'rkrk 4/14/2007 01:43'!
loadLevel: aBreakoutLevel 	

	level _ aBreakoutLevel.
	self 
		reset;
		showText: aBreakoutLevel name forSeconds: 4;
		addBlocks: level blocks inColumns: level columns andRows: level rows;
		addRacketsAtCriticalSides: level criticalGameFieldSides;
		newBall;
		texture: level textureDirectory.
	owner update.! !


!BreakoutGameField methodsFor: 'stepping and presenting' stamp: 'rkrk 4/14/2007 01:43'!
showNothing

	self removeMorph: textMorph! !

!BreakoutGameField methodsFor: 'stepping and presenting' stamp: 'rkrk 4/14/2007 01:43'!
showText: aString 

	self addMorphCentered: (textMorph contents: aString; font: Breakout defaultFontBig; yourself)! !

!BreakoutGameField methodsFor: 'stepping and presenting' stamp: 'rkrk 4/14/2007 01:43'!
showText: aString forSeconds: seconds.

	[self showText: aString.
	(Delay forSeconds: seconds) wait.
	self showNothing.] 
		forkAt: Processor highIOPriority! !

!BreakoutGameField methodsFor: 'stepping and presenting' stamp: 'rkrk 4/14/2007 01:43'!
step

	self gameElements do: [:gameElement | gameElement move].! !

!BreakoutGameField methodsFor: 'stepping and presenting' stamp: 'rkrk 4/16/2007 00:09'!
stepTime

	^ 0! !


!BreakoutGameField methodsFor: 'submorphs' stamp: 'rkrk 4/14/2007 01:43'!
addGameElement: aMorph.

	(aMorph isKindOf: BreakoutGameElement) 
		ifTrue: [gameElements add: aMorph]! !

!BreakoutGameField methodsFor: 'submorphs' stamp: 'rkrk 4/14/2007 01:43'!
addMorph: aMorph

	super addMorph: aMorph.
	self addGameElement: aMorph.! !

!BreakoutGameField methodsFor: 'submorphs' stamp: 'rkrk 4/14/2007 01:43'!
addMorphBack: aMorph

	super addMorphBack: aMorph.
	self addGameElement: aMorph.! !

!BreakoutGameField methodsFor: 'submorphs' stamp: 'rkrk 4/14/2007 01:43'!
removeAllMorphs

	super removeAllMorphs.
	gameElements := OrderedCollection with: self! !

!BreakoutGameField methodsFor: 'submorphs' stamp: 'rkrk 4/14/2007 01:43'!
removeMorph: aMorph

	super removeMorph: aMorph.
	gameElements remove: aMorph ifAbsent: []! !


!BreakoutGameField methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
isBreakoutGameField

	^ true! !


!BreakoutGameField methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 01:43'!
asGrayScale

	self fillStyle: ((BitmapFillStyle fromForm: self imageForm asGrayScale) 
			origin: self position)! !

!BreakoutGameField methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 01:43'!
textureDirectory

	^ level textureDirectory! !

!BreakoutGameField methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 01:43'!
textureName

	^ 'background.gif'! !


BreakoutBlock subclass: #BreakoutHardBlock
	instanceVariableNames: 'wasHit'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutHardBlock commentStamp: 'rkrk 1/31/2007 01:12' prior: 0!
Instances have to be hit two times.!
]style[(35)f1!


!BreakoutHardBlock methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
hit: aGameElement

	wasHit 
		ifTrue: [super hit: aGameElement]
		ifFalse: [wasHit := true.
			self texture: owner textureDirectory]! !


!BreakoutHardBlock methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
initialize

	super initialize.
	wasHit := false! !


!BreakoutHardBlock methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
score

	^ 100! !


!BreakoutHardBlock methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 01:43'!
textureName

	wasHit 
		ifTrue: [^'hardBlockHit.gif']
		ifFalse: [^'hardBlock.gif']! !


RectangleMorph subclass: #BreakoutHighscore
	instanceVariableNames: 'highscore newestScore'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!

!BreakoutHighscore methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 02:48'!
addHighScoreWithQuery: score

	| playerName |
	(self scoreCanBeAdded: score) ifFalse: [^ self].	
	playerName := (FillInTheBlankMorph
		request: 'Congratulations!!, ', Character cr asString, 
		'You are in the top ten of the best Breakout players!!', Character cr asString, 
		'Please enter your name'
		initialAnswer: 'Anonymous'
		centerAt: Display center).		
	self addHighscore: score name: playerName! !

!BreakoutHighscore methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 14:51'!
addHighscore: aNumber name: aString	

	newestScore := aNumber -> aString.
	highscore  
		add: newestScore;
		reSort.
	highscore := self highscore first: (self numberOfScoresToSave min: self highscore size).
	self updateSubmorphs! !

!BreakoutHighscore methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
highscore

	^ highscore 
		sortBlock: [:x :y | x key >= y key];
		yourself! !

!BreakoutHighscore methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 02:45'!
scoreCanBeAdded: score

	^ score > self lowestScore or: [ self highscore size < self numberOfScoresToSave ]! !


!BreakoutHighscore methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 02:03'!
initialize

	super initialize.
	highscore:= SortedCollection sortBlock: [:x :y | x key >= y key].
	self 
		makeLayout;
		updateSubmorphs! !

!BreakoutHighscore methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
makeLayout

	self
	 	extent: 200@310;
		layoutPolicy: TableLayout new;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap;
		fillStyle: (Breakout defaultFillStyleFor: self);
		cellInset: 5;
		layoutInset: 5.! !


!BreakoutHighscore methodsFor: 'private' stamp: 'rkrk 4/14/2007 14:47'!
lowestScore

	^ self highscore 
		ifEmpty: [0]
		ifNotEmpty: [self highscore last key]! !

!BreakoutHighscore methodsFor: 'private' stamp: 'rkrk 4/14/2007 01:43'!
numberOfScoresToSave

	^ 10! !

!BreakoutHighscore methodsFor: 'private' stamp: 'rkrk 4/15/2007 23:57'!
updateSubmorphs

	self 
		removeAllMorphs;
		addMorph: ((StringMorph contents: 'TOP TEN') font: Breakout defaultFontBig).
	highscore do: [:scoreAndName | | string |
		string := ((StringMorph contents: scoreAndName key asString, ' - ', scoreAndName value)
			font: Breakout defaultFont).
		scoreAndName = newestScore ifTrue: [string color: Color blue].
		self addMorphBack: string.].! !


Object subclass: #BreakoutLevel
	instanceVariableNames: 'rows columns textureDirectory blocks criticalSides name'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutLevel commentStamp: 'rkrk 1/31/2007 00:56' prior: 0!
BreakoutLevel contains informations and morphs for loading a level
	- blocks
	- criticalGameFieldSides
	- ball
	- textureDireactory
	...!


!BreakoutLevel methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
ball

	^ BreakoutBall new texture: self textureDirectory! !

!BreakoutLevel methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
blocks

	
	^ blocks collect: [:each | 
		each ifNotNil: [ | copy |
			copy := each copy.
			copy texture: self textureDirectory.]]! !

!BreakoutLevel methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
columns

	^ columns asInteger! !

!BreakoutLevel methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
criticalGameFieldSides

	^ criticalSides! !

!BreakoutLevel methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
racketForSide: side	

	(side = #left or: [side = #right])
		ifTrue: [^ BreakoutRacketVertical new texture: self textureDirectory].
	(side = #top or: [side = #bottom])
		ifTrue:[^ BreakoutRacketHorizontal new texture: self textureDirectory]! !

!BreakoutLevel methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
rows

	^ rows asInteger! !

!BreakoutLevel methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
textureDirectory

	^ textureDirectory ! !


!BreakoutLevel methodsFor: 'configuration' stamp: 'rkrk 4/14/2007 01:43'!
blocks: anOrderedCollection

	blocks := anOrderedCollection.! !

!BreakoutLevel methodsFor: 'configuration' stamp: 'rkrk 4/14/2007 01:43'!
columns: aNumber 

	columns := aNumber! !

!BreakoutLevel methodsFor: 'configuration' stamp: 'rkrk 4/14/2007 01:43'!
criticalGameFieldSides: anOrderedCollection

	criticalSides := anOrderedCollection! !

!BreakoutLevel methodsFor: 'configuration' stamp: 'rkrk 4/14/2007 01:43'!
name: aString

	name := aString! !

!BreakoutLevel methodsFor: 'configuration' stamp: 'rkrk 4/14/2007 01:43'!
rows: aNumber 

	rows := aNumber! !

!BreakoutLevel methodsFor: 'configuration' stamp: 'rkrk 4/14/2007 01:43'!
textureDirectory: aDirectory

	textureDirectory := aDirectory asString.! !


!BreakoutLevel methodsFor: 'initializing' stamp: 'rkrk 4/14/2007 01:43'!
default

	"load a default level"
	rows := 15.
	columns := 15.
	blocks := OrderedCollection new.
	criticalSides := #(#bottom).
	name:= 'aLevel'.
	textureDirectory := ''! !

!BreakoutLevel methodsFor: 'initializing' stamp: 'rkrk 4/14/2007 01:43'!
initialize

	self default! !


!BreakoutLevel methodsFor: 'private' stamp: 'rkrk 4/14/2007 01:43'!
gameElementsDo: aBlock 

	blocks do: aBlock! !

!BreakoutLevel methodsFor: 'private' stamp: 'rkrk 4/14/2007 01:43'!
updateTextureOfGameElements

	self gameElementsDo: [:element |
		element ifNotNil: [element texture: self textureDirectory]]! !


!BreakoutLevel methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
name

	^ name! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BreakoutLevel class
	instanceVariableNames: ''!

!BreakoutLevel class methodsFor: 'instance creation' stamp: 'rkrk 1/25/2007 02:28'!
readFrom: textStringOrStream
	^ (Object readFrom: textStringOrStream) updateTextureOfGameElements! !


RectangleMorph subclass: #BreakoutQuestioner
	instanceVariableNames: 'textMorph done result'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutQuestioner commentStamp: 'rkrk 1/31/2007 01:13' prior: 0!
Works like the FillInTheBlanksMorph but returnes only true and false.

"
BreakoutQuestioner ask: 'Is Breakout a fine game?'
"!


!BreakoutQuestioner methodsFor: 'private' stamp: 'rkrk 4/14/2007 01:43'!
answer: aBool

	self delete.
	result := aBool! !


!BreakoutQuestioner methodsFor: 'accessing' stamp: 'rkrk 4/17/2007 02:33'!
ask
	| world |
	owner 
		ifNil: [world := self outermostWorldMorph. world addMorph: self]
		ifNotNil: [world := self world].
	[done] whileFalse: [world doOneCycle].
	^ result! !

!BreakoutQuestioner methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 01:43'!
question: aString

	textMorph contents: aString! !


!BreakoutQuestioner methodsFor: 'submorphs-add/remove' stamp: 'rkrk 4/14/2007 01:43'!
addButtonWithLabel: label arguments: arguments

	self	addMorphBack: (IconicButton new
		target: self;
		actionSelector: #answer:;
		arguments: arguments;
		labelGraphic: label;
		extent: 70@70;
		color: Color transparent;
		borderWidth: 0;
		yourself)! !

!BreakoutQuestioner methodsFor: 'submorphs-add/remove' stamp: 'rkrk 4/14/2007 01:43'!
addText		

	self	addMorph: (textMorph := StringMorph new font: Breakout defaultFontBig)! !

!BreakoutQuestioner methodsFor: 'submorphs-add/remove' stamp: 'rkrk 4/14/2007 01:43'!
delete

	done := true.
	super delete! !


!BreakoutQuestioner methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
initialize

	super initialize.
	done _ false.
	result _ false.
	self 
		makeLayout;
		makeBackground; 
		addText; 
		addButtonWithLabel: self class yesIcon arguments: #(true);
		addButtonWithLabel: self class noIcon arguments: #(false)! !

!BreakoutQuestioner methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
makeBackground

	self fillStyle: (Breakout defaultFillStyleFor: self)! !

!BreakoutQuestioner methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
makeLayout

	self 
		extent:  460@120;
		layoutPolicy: TableLayout new;
		listDirection: #lefToRight;
		cellInset: 20;
		layoutInset: 20;
		hResizing: #shrinkWrap;
		vResizing: #shrinkWrap.! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BreakoutQuestioner class
	instanceVariableNames: ''!

!BreakoutQuestioner class methodsFor: 'icons' stamp: 'rkrk 1/28/2007 22:22'!
noIcon
	^ (Form readFrom:   '(Form
	extent: 21@15
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 0 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 0 4294385920 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 0 0 0 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 0 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 0 0 4294385920 4294385920 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 0 4294385920 4294385920 4294385920 4294385920 0 0 0 0 0 0 0 4294385920 0 0 0 0 0 0 0 0 0 0 4294385920 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)')
	scaledToSize: 70@70.! !

!BreakoutQuestioner class methodsFor: 'icons' stamp: 'rkrk 1/28/2007 22:20'!
yesIcon
	^ (Form readFrom:  '(Form
	extent: 19@14
	depth: 32
	fromArray: #( 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4278777600 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 4278777600 0 0 0 0 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 0 0 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 0 0 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 0 4278777600 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 0 0 0 0 0 0 4278777600 4278777600 4278777600 4278777600 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 0 4278777600 4278777600 0 0 0 0 0 0 0 0 0 0 0)
	offset: 0@0)')
	scaledToSize: 70@70.! !


BreakoutGameElement subclass: #BreakoutRacket
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!

!BreakoutRacket methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
initialize

	super initialize.
	self
		color: Color red;
		setFillStyleFromForm: self imageForm! !


!BreakoutRacket methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
degree

	^ super degree rounded! !

!BreakoutRacket methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
degree: deg 

	deg \\ 90 = 0
		ifTrue: [super degree: deg.
			self direction: self direction rounded]! !

!BreakoutRacket methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
ensureToBeInGameField

	owner ifNotNil: [self bounds: (self bounds translatedToBeWithin: owner bounds)]
		! !

!BreakoutRacket methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:47'!
move: aCollisionSet 

	aCollisionSet	do: [:element | 
		element hit: self.								
		element isBreakoutRacket ifTrue: [self 
			direction: self direction negated;
			speedToZero]].
	self 
		calculateNewPosition;
		ensureToBeInGameField;
		increaseMoveProgress
		! !

!BreakoutRacket methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
moveAmount

	^ 15! !

!BreakoutRacket methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
moveBackward

	self subclassResponsibility! !

!BreakoutRacket methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
moveForward

	self subclassResponsibility! !

!BreakoutRacket methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
speedToDefault

	^ self speed: self moveAmount! !


!BreakoutRacket methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
isBreakoutRacket

	^ true! !


!BreakoutRacket methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 01:43'!
textureName

	^ 'racket.gif'! !


!BreakoutRacket methodsFor: 'event handling' stamp: 'rkrk 4/14/2007 01:43'!
keyReleased

	self speedToZero! !


!BreakoutRacket methodsFor: 'as yet unclassified' stamp: 'rkrk 4/14/2007 01:43'!
hit: aGameElement

	"if aGameElement is caught in me, throw it out"
	(self bounds containsPoint: aGameElement center)
		ifTrue: [| distance pointOnBoundingBox positionOutOfMe |
			pointOnBoundingBox := self bounds pointNearestTo: aGameElement center.
			positionOutOfMe := 2*(pointOnBoundingBox  - aGameElement center).
			aGameElement position: aGameElement position + positionOutOfMe.]! !


BreakoutRacket subclass: #BreakoutRacketHorizontal
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!

!BreakoutRacketHorizontal methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
initialize

	super initialize.
	self 
		degree: 0;
		extent: 100@20.! !


!BreakoutRacketHorizontal methodsFor: 'event handling' stamp: 'rkrk 4/4/2007 00:57'!
keyPressedCharacter: char

	(char = Character arrowLeft or: [char = ${])
		ifTrue: [self moveForward].
	(char = Character arrowRight or: [char = $|])
		ifTrue: [self moveBackward]! !


!BreakoutRacketHorizontal methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
moveBackward

	self 
		degree: 0;
		speedToDefault.! !

!BreakoutRacketHorizontal methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
moveForward

	self 
		degree: 180;
		speedToDefault.! !


BreakoutRacket subclass: #BreakoutRacketVertical
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!

!BreakoutRacketVertical methodsFor: 'initialization' stamp: 'rkrk 4/14/2007 01:43'!
initialize

	super initialize.
	self 
		degree: 0;
		extent: 20@100.
	self fillStyle: (self fillStyle
		direction: 0 @ self height;
		normal: self width @ 0;
		yourself)! !


!BreakoutRacketVertical methodsFor: 'event handling' stamp: 'rkrk 4/4/2007 00:57'!
keyPressedCharacter: char

	(char = Character arrowUp or: [char = $~])
		ifTrue: [self moveForward].
	(char = Character arrowDown or: [char = $}])
		ifTrue: [self moveBackward].
! !


!BreakoutRacketVertical methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
moveBackward

	self 
		degree: 90;
		speedToDefault.! !

!BreakoutRacketVertical methodsFor: 'moving' stamp: 'rkrk 4/14/2007 01:43'!
moveForward

	self 
		degree: 270;
		speedToDefault.! !


!BreakoutRacketVertical methodsFor: 'as yet unclassified' stamp: 'rkrk 4/14/2007 01:43'!
setFillStyleFromForm: aForm

"the texture has to be turned because this racket is vertical"
	super setFillStyleFromForm: aForm.
	self fillStyle: (self fillStyle
		direction: 0@ self height;
		normal: self width @ 0;
		yourself)! !


Object subclass: #BreakoutResources
	instanceVariableNames: ''
	classVariableNames: 'Resources'
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutResources commentStamp: 'rkrk 4/3/2007 14:52' prior: 0!
This class provides a global interface for getting all the images and level informations.!


"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

BreakoutResources class
	instanceVariableNames: ''!

!BreakoutResources class methodsFor: 'images' stamp: 'rkrk 4/25/2007 21:24'!
getLabel: label
	^ Resources at: label , '.gif'! !

!BreakoutResources class methodsFor: 'images' stamp: 'rkrk 4/25/2007 21:24'!
getLifeDisplayLifeCount: aNumber
	^ Resources at: 'balldisplay' , aNumber asString , '.gif'! !

!BreakoutResources class methodsFor: 'images' stamp: 'rkrk 4/25/2007 21:24'!
getTextureNamed: aString
	^ Resources at: aString ifAbsent: nil
	! !

!BreakoutResources class methodsFor: 'images' stamp: 'rkrk 4/25/2007 21:24'!
getTitle
	^ Resources at: 'breakout.gif'
	! !


!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 4/13/2007 23:57'!
defaultDirectoryName

	^ 'breakout', FileDirectory slash! !

!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 4/3/2007 16:36'!
directory

	^ FileDirectory on: (FileDirectory default fullNameFor: self defaultDirectoryName)! !

!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 4/3/2007 17:50'!
formFromFileNamed: fileName
	
	| form |
	form := ImageReadWriter formFromStream: (self streamFromFileNamed: fileName).
	form mapColor: Color white to: Color transparent.
	^ form! !

!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 4/3/2007 17:24'!
isImage: aFileName

	^ ImageReadWriter allTypicalFileExtensions anySatisfy: [:ext | aFileName includesSubString: ext]! !

!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 4/4/2007 01:21'!
isLevel: fileName

	^ 'level*' match: fileName! !

!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 4/4/2007 01:22'!
levelFromFileNamed: fileName

	^ BreakoutLevel readFrom: (self streamFromFileNamed: fileName)! !

!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 4/13/2007 23:57'!
readFileNamed: fileName 

	(self isImage: fileName) ifTrue: [^ self formFromFileNamed: fileName].
	(self isLevel: fileName) ifTrue: [^ self levelFromFileNamed: fileName].
	^ (self streamFromFileNamed: fileName) contents! !

!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 11/28/2007 15:00'!
readFiles

	self directory fileNames do: [:fileName | fileName first = $. ifFalse: [Resources at: fileName put: (self readFileNamed: fileName)]]! !

!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 4/25/2007 21:25'!
readThemes
	self directory directoryNames do: [:directoryName | | themeDir |
		('*theme*' match: directoryName) 
			ifTrue: [themeDir := FileDirectory on: (self directory fullNameFor: directoryName).
					themeDir fileNames do: [:fileName | | aColorForm path |
						path := directoryName , FileDirectory slash , fileName.
						aColorForm := self formFromFileNamed: path.						
						Resources at: path put: aColorForm]]]! !

!BreakoutResources class methodsFor: 'fileIn/Out' stamp: 'rkrk 4/13/2007 23:58'!
streamFromFileNamed: fileOrDirectory

	| pathName relativeDirEntries file path directory |
	pathName := self defaultDirectoryName.
	relativeDirEntries := (fileOrDirectory findTokens: '\/').
	relativeDirEntries allButLastDo: [:dir | pathName := pathName, dir, FileDirectory slash].
	file := relativeDirEntries last.
	path := FileDirectory default fullPathFor: pathName.
	directory := FileDirectory on: path.
	(directory fileExists: file)
		ifTrue: [^ directory fileNamed: file]
		ifFalse: [^ nil]! !


!BreakoutResources class methodsFor: 'initializing' stamp: 'rkrk 4/25/2007 21:25'!
initialize
	Resources := Dictionary new.
	self 
		readFiles
		readThemes! !


!BreakoutResources class methodsFor: 'level' stamp: 'rkrk 4/4/2007 01:07'!
defaultNameForLevel: aLevelNumber
	^ 'level' , aLevelNumber asString! !

!BreakoutResources class methodsFor: 'level' stamp: 'rkrk 4/25/2007 21:25'!
getLevel: aNumber

	^ Resources at: (self defaultNameForLevel: aNumber) ifAbsent: [BreakoutLevel new]! !


!BreakoutResources class methodsFor: 'highscore' stamp: 'rkrk 4/25/2007 21:23'!
getHighscore

	^ Resources at: #highscore ifAbsent: [BreakoutHighscore new].! !

!BreakoutResources class methodsFor: 'highscore' stamp: 'rkrk 4/25/2007 21:25'!
saveHighscore: newHighscore

	Resources at: #highscore put: newHighscore.! !


BreakoutBlock subclass: #BreakoutStoneBlock
	instanceVariableNames: ''
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!BreakoutStoneBlock commentStamp: 'rkrk 1/31/2007 01:13' prior: 0!
Instances are undestroyable.!
]style[(28)f1!


!BreakoutStoneBlock methodsFor: 'collision handling' stamp: 'rkrk 4/14/2007 01:43'!
hit: aGameElement! !


!BreakoutStoneBlock methodsFor: 'testing' stamp: 'rkrk 4/14/2007 01:43'!
isDestroyable

	^ false! !


!BreakoutStoneBlock methodsFor: 'texturizing' stamp: 'rkrk 4/14/2007 01:43'!
textureName

	^ 'stoneBlock.gif'! !


OrderedCollection subclass: #Rasterizer
	instanceVariableNames: 'rectangle rows columns'
	classVariableNames: ''
	poolDictionaries: ''
	category: 'Breakout-Game'!
!Rasterizer commentStamp: 'rkrk 1/5/2007 01:23' prior: 0!
Instances from this class take a Rectangle and divide it evenly conforming the given columns and rows. You can access the positions of every cell via the standard operations of an OrderedCollection (beginning with the positions top left line-by-line).!


!Rasterizer methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 02:53'!
cellHeight

	^ rectangle extent y / (rows max: 1)
	! !

!Rasterizer methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 02:53'!
cellWidth

	^ rectangle extent x / (columns max: 1)! !

!Rasterizer methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 02:53'!
columns

	^ columns! !

!Rasterizer methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 02:53'!
height

	^ rectangle height! !

!Rasterizer methodsFor: 'accessing' stamp: 'rkrk 4/15/2007 23:57'!
rasterize

	0 to: rows - 1 do:
		[:y | 0 to: columns - 1 do:
			[:x | self add: (x * self cellWidth @ (y * self cellHeight) translateBy: rectangle origin)]]! !

!Rasterizer methodsFor: 'accessing' stamp: 'rkrk 4/14/2007 02:53'!
width

	^rectangle width! !


!Rasterizer methodsFor: 'private' stamp: 'rkrk 4/14/2007 02:53'!
cellPositionFromCoordinates: point

	"answers the coordinates of the topLeft position of the cell to which the
	points belongs"
	^ self detect: [:position | 
		(position corner: position + (self cellWidth @ self cellHeight)) containsPoint: point]! !

!Rasterizer methodsFor: 'private' stamp: 'rkrk 4/14/2007 02:53'!
columns: aNumber

	columns := aNumber! !

!Rasterizer methodsFor: 'private' stamp: 'rkrk 4/14/2007 02:53'!
indexFromCoordinates: point 

	"answers the index of the cell to which the points
	belongs."
	^ self indexOf: (self cellPositionFromCoordinates: point)! !

!Rasterizer methodsFor: 'private' stamp: 'rkrk 4/14/2007 02:54'!
rectangle: aRectangle

	rectangle := aRectangle! !

!Rasterizer methodsFor: 'private' stamp: 'rkrk 4/14/2007 02:54'!
rows: aNumber

	rows := aNumber! !

"-- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- -- "!

Rasterizer class
	instanceVariableNames: ''!

!Rasterizer class methodsFor: 'creation' stamp: 'rkrk 1/5/2007 01:36'!
new
	self error: 'New is not supported. Use #rasterize:withColumns:andRows: instead'! !

!Rasterizer class methodsFor: 'creation' stamp: 'rkrk 4/3/2007 11:31'!
rasterize: aRectangle withColumns: columns andRows: rows 
	^ super new 
		rectangle: aRectangle;
		columns: columns;
		rows: rows;
		rasterize;
		yourself! !

BreakoutResources initialize!